from .base import Rule
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type
from .basic import *
from .boolean import AcceptedRule, DeclinedRule

class RequiredIfRule(RequiredRule):
    _count_parameter = 1
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:  
            expected_value = self.get_field_value(params[0], False)
            conditions = params[1:]
            
            condition_met = expected_value in conditions
                
        if condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class RequiredUnlessRule(RequiredRule):
    _count_parameter = 2
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:  
            expected_value = self.get_field_value(params[0], False)
            conditions = params[1:]
            
            condition_met = expected_value in conditions
                
        if not condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class RequiredWithRule(RequiredRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        condition = [f in self.validator.data and not self.is_empty(self.get_field_value(f)) for f in params]
        if any(condition):
            return super().validate(field, value, params)
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements

class RequiredWithAllRule(RequiredRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        condition = [f in self.validator.data and not self.is_empty(self.get_field_value(f)) for f in params]
        
        if all(condition):
            return super().validate(field, value, params)
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements
    
class RequiredWithoutRule(RequiredRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if any(f not in self.validator.data for f in params):
            return super().validate(field, value, params)
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements
    
class RequiredWithoutAllRule(RequiredRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if all(f not in self.validator.data for f in params):
            return super().validate(field, value, params)
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements
    
class RequiredIfAcceptedRule(RequiredRule, AcceptedRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if AcceptedRule.validate(self, field, params[0], params):
            return super().validate(field, value, params)
        
        return True
    
class RequiredIfDeclinedRule(RequiredRule, DeclinedRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if DeclinedRule.validate(self, field, params[0], params):
            return super().validate(field, value, params)
        
        return True
    
class RequiredArrayKeysRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, dict):
            return False
        return all(key in value for key in params)
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must contain all required keys: :values"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            'values': ', '.join(self._params),
        }
        
        return replacements
    
class ProhibitedIfRule(ProhibitedRule):
    _count_parameter = 1
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:    
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            condition_met = expected_value in conditions
        
        if condition_met:
            return super().validate(field, value, params)
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class ProhibitedUnlessRule(ProhibitedRule):
    _count_parameter = 2
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:  
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            condition_met = expected_value in conditions
        
        if not condition_met:
            return super().validate(field, value, params)
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class ProhibitedIfAcceptedRule(ProhibitedRule, AcceptedRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if AcceptedRule.validate(self, field, params[0], params):
            return super().validate(field, value, params)
        return True

class ProhibitedIfDeclinedRule(ProhibitedRule, DeclinedRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if DeclinedRule.validate(self, field, params[0], params):
            return super().validate(field, value, params)
        
        return True
    
class PresentIfRule(PresentRule):
    _count_parameter = 1
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            
            condition_met = expected_value in conditions
            
        if condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class PresentUnlessRule(PresentRule):
    _count_parameter = 2
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            
            condition_met = expected_value in conditions
            
        if not condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class PresentWithRule(PresentRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if any(self.get_field_value(param, None) is not None for param in params):
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements
    
class PresentWithAllRule(PresentRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if all(self.get_field_value(param, None) is not None for param in params):
            return super().validate(field, value, params)
        
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements

class MissingIfRule(MissingRule):
    _count_parameter = 2
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            condition_met = expected_value in conditions
            
        if condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
    
class MissingUnlessRule(MissingRule):
    _count_parameter = 2
    _accept_closure = True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            condition_met = expected_value in conditions
            
        if not condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':values': ', '.join(self._params[1:]),
        }
        
        return replacements
            
class MissingWithRule(MissingRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not params:
            return False
        
        if any(self.get_field_value(param, None) is not None for param in params):
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements
    
class MissingWithAllRule(MissingRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not params:
            return False
        
        if all(self.get_field_value(param, None) is not None for param in params):
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':others': ', '.join(self._get_display_name(self._params)),
        }
        
        return replacements
    
class ExcludeIfRule(ExcludeRule):
    _count_parameter = 1
    _accept_closure =  True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            condition_met = expected_value in conditions
        
        if condition_met:
            return super().validate(field, value, params)
        return True

class ExcludeUnlessRule(ExcludeRule):
    _count_parameter = 2
    _accept_closure =  True
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif isinstance(params[0], bool):
            condition_met = params[0]
        else:
            expected_value, conditions = self.get_field_value(params[0], None), params[1:]
            condition_met = expected_value in conditions
        
        if not condition_met:
            return super().validate(field, value, params)
        return True

class ExcludeWithRule(ExcludeRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if any(not self.is_empty(self.get_field_value(param, None)) for param in params):
            return super().validate(field, value, params)
            
        return True
    
class ExcludeWithoutRule(ExcludeRule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if any(self.is_empty(self.get_field_value(param, None)) for param in params):
            return super().validate(field, value, params)
            
        return True



from .base import Rule
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type
import decimal

class NumericRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if isinstance(value, (int, float)):
            return True
        if not isinstance(value, str):
            return False
        return value.replace('.', '', 1).isdigit()
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a number."
    
class IntegerRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if isinstance(value, int):
            return True
        if not isinstance(value, str):
            return False
        return value.isdigit()
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be an integer."

class DigitsRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            digits = int(params[0])
        except ValueError:
            return False
            
        return value.isdigit() and len(value) == digits
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be :value digits."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class DigitsBetweenRule(Rule):
    _count_parameter = 2
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            min_digits = int(params[0])
            max_digits = int(params[1])
        except ValueError:
            return False
            
        return value.isdigit() and min_digits <= len(value) <= max_digits
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be between :min and :max digits."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':min': self._params[0],
            ':max': self._params[1],
        }
        
        return replacements
    
class DecimalRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            decimal.Decimal(str(value))
            return True
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return "The :attribute must be a decimal number."

class GreaterThanRule(Rule):
    _name = 'gt'
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            threshold = decimal.Decimal(self.get_field_value(params[0], params[0]))
            numeric_value = decimal.Decimal(str(value))
            return numeric_value > threshold
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be greater than :value."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class GreaterThanOrEqualRule(Rule):
    _name = 'gte'
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            threshold = decimal.Decimal(self.get_field_value(params[0], params[0]))
            numeric_value = decimal.Decimal(str(value))
            return numeric_value >= threshold
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be greater than or equal to :value."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class LessThanRule(Rule):
    _name = 'lt'
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            threshold = decimal.Decimal(self.get_field_value(params[0], params[0]))
            numeric_value = decimal.Decimal(str(value))
            return numeric_value < threshold
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be less than :value."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class LessThanOrEqualRule(Rule):
    _name = 'lte'
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            threshold = decimal.Decimal(self.get_field_value(params[0], params[0]))
            numeric_value = decimal.Decimal(str(value))
            return numeric_value <= threshold
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be less than or equal to :value."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class MaxDigitsRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            max_digits = int(params[0])
            numeric_value = decimal.Decimal(str(value))
            str_value = str(numeric_value).replace("-", "")
            if '.' in str_value:
                str_value = str_value.replace(".", "")
            return len(str_value) <= max_digits
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must not exceed :value digits."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class MinDigitsRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            min_digits = int(params[0])
            numeric_value = decimal.Decimal(str(value))
            str_value = str(numeric_value).replace("-", "")
            if '.' in str_value:
                str_value = str_value.replace(".", "")
            return len(str_value) >= min_digits
        except (decimal.InvalidOperation, TypeError, ValueError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must have at least :value digits."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0],
        }
        
        return replacements

class MultipleOfRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            divisor = float(params[0])
            if divisor == 0:
                return False
            num = float(value)
            return num % divisor == 0
        except (ValueError, TypeError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a multiple of :value."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self._params[0]
        }
        
        return replacements


from validator.factory import RuleFactory
from validator.services.rule_conflict import RuleConflictChecker
from validator.rules import Rule
from typing import Dict, List, Union, Tuple
from validator.exceptions import RuleNotFoundException
from collections.abc import Iterable, Sequence
from numbers import Number
from enum import Enum
import inspect
import warnings

class RulePreparer:
    """Handles rule preparation and parsing"""
    def __init__(self, rule_factory: RuleFactory):
        self.rule_factory = rule_factory
        self.added_rules = {}

    def prepare(self, raw_rules: Dict[str, Union[str, List[Union[str, Rule]]]]) -> Dict[str, List[Rule]]:
        """Convert raw rules to prepared validation rules"""
        prepared_rules = {}
        for field, rule_input in raw_rules.items():
            if not rule_input:
                continue
            
            prepared_rules[field] = self._generate_rule(field, rule_input)
                
        return prepared_rules
    
    def _generate_rule(self, field, rule_input):
        rules = self._convert_to_rules(rule_input)
        RuleConflictChecker.check_conflicts(rules)
        
        return self._deduplicate_rules(field, rules)

    def _convert_to_rules(self, rule_input: Union[str, List[Union[str, Rule]], Rule, Tuple[Union[str, Rule], str], Tuple[Union[str, Rule], str]]) -> List[Rule]:
        """Convert mixed rule input to list of Rule objects"""
        if rule_input is None:
            return []
        if isinstance(rule_input, Rule):
            return [rule_input]
        if isinstance(rule_input, list | tuple):
            rules = []
            for r in rule_input:
                if isinstance(r, str):
                    rules.extend(self._parse_rule_string(r))
                elif isinstance(r, Rule):
                    rules.append(r)
                elif isinstance(r, (tuple, list)):
                    if len(r) == 0:
                        continue
                    try:
                        # Handle tuple/list format
                        rule_name = r[0]
                        params = r[1:] if len(r) > 1 else []
                        params = self._parse_params(params)
                        
                        if isinstance(rule_name, str):
                            rule = RuleFactory.create_rule(rule_name)
                            rule.params = params
                        elif isinstance(rule_name, Rule):
                            rule = rule_name
                            if params:
                                warnings.warn(f"Parameters {params} are ignored for Rule instance")
                        else:
                            raise ValueError(f"Invalid rule name type in {r}")
                            
                        rules.append(rule)
                    except Exception as e:
                        raise ValueError(f"Invalid rule format {r}: {str(e)}")   
            return rules
        if isinstance(rule_input, str):
            return self._parse_rule_string(rule_input)
        raise ValueError(f"Invalid rule input type: {type(rule_input)}")

    def _parse_rule_string(self, rule_str: str) -> List[Rule]:
        """Parse rule string into Rule objects"""
        rules = []
        for rule_part in rule_str.split('|'):
            rule_part = rule_part.strip()
            if not rule_part:
                continue
            rule_name, params = self._parse_rule_part(rule_part)
            try:
                rule = self.rule_factory.create_rule(rule_name)
                if params:
                    rule.params = params
                rules.append(rule)
            except RuleNotFoundException as e:
                warnings.warn(str(e))
        return rules

    def _parse_rule_part(self, rule_part: str) -> Tuple[str, List[str]]:
        """Parse single rule part into name and parameters"""
        if ':' not in rule_part:
            return rule_part, []
        rule_name, param_str = rule_part.split(':', 1)
            
        if rule_name in {'regex', 'not_regex', 'dimensions'}:
            return rule_name, [param_str]
        return rule_name, [p.strip() for p in param_str.split(',') if p.strip()]
    
    def _parse_params(self, params: Union[Tuple, List, str, Enum]):
        return set(tuple(x) if isinstance(x, list) else x for x in params)

    def _deduplicate_rules(self, field: str, rules: List[Rule]) -> List[Rule]:
        """Remove duplicate rules based on name and parameters"""
        unique = []
        for rule in rules:
            identifier = (rule.rule_name, tuple(rule.params))
            if rule.rule_name != 'when':
                if identifier not in self.added_rules.get(field, []):
                    if isinstance(self.added_rules.get(field), set):
                        self.added_rules[field].add(identifier)
                    else:
                        self.added_rules[field] = set()
                        self.added_rules[field].add(identifier)
                        
                    unique.append(rule)
            else:
                unique.append(rule)
        return unique


from .base import Rule
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type

class BooleanRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if isinstance(value, bool):
            return True
        if isinstance(value, str):
            return value.lower() in ['true', 'false', '1', '0', 'yes', 'no', 'on', 'off', 'False', 'True']
        if isinstance(value, int):
            return value in [0, 1]
        return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute field must be true or false."
    
class AcceptedRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        value = self.get_field_value(value, value)
        
        if isinstance(value, str):
            return value.lower() in ['yes', 'on', '1', 1, True, 'true', 'True']
        if isinstance(value, int):
            return value == 1
        if isinstance(value, bool):
            return value
        return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be accepted."

class DeclinedRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        value = self.get_field_value(value, value)
        
        if isinstance(value, str):
            return value.lower() in ['no', 'off', '0', 0, False, 'false', 'False']
        if isinstance(value, int):
            return value == 0
        if isinstance(value, bool):
            return not value
        return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be declined."
    
class AcceptedIfRule(AcceptedRule):
    _count_parameter = 2
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        condition_met = False

        if len(params) == 1 and callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif len(params) == 1 and isinstance(params[0], bool):
            condition_met = params[0]
        else:  
            expected_value = self.get_field_value(params[0], None)
            condition = params[1]
            
            if expected_value == condition:
                condition_met = True
            
        if condition_met:
            return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':value': self._params[1],
        }
        
        return replacements
    
class AcceptedUnlessRule(AcceptedRule):
    _count_parameter = 2
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        other_field, other_value = params[0], params[1]
        actual_value = self.get_field_value(other_field, None) 
        
        if actual_value == other_value:
            return True
            
        return super().validate(field, value, params)
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':value': self._params[1],
        }
        
        return replacements
    
class DeclinedIfRule(DeclinedRule):
    _count_parameter = 2
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        condition_met = False
                
        if len(params) == 1 and callable(params[0]):
            condition_met = params[0](self.validator.data)
        elif len(params) == 1 and isinstance(params[0], bool):
            condition_met = params[0]
        else:  
            expected_value = self.get_field_value(params[0], None)
            condition = params[1]
            
            if expected_value == condition:
                condition_met = True
            
            if condition_met:
                return super().validate(field, value, params)
        
        return True
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':value': self._params[1],
        }
        
        return replacements
    
class DeclinedUnlessRule(DeclinedRule):
    _count_parameter = 2
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if len(params) < 2:
            return False
            
        other_field, other_value = params[0], params[1]
        actual_value = self.get_field_value(other_field, '') 
        
        if actual_value == other_value:
            return True
            
        return super().validate(field, value, params)
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':other': self._get_display_name(self._params[0]),
            ':value': self._params[1],
        }
        
        return replacements